/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include "sessionparser.h"
#include "log.h"
#include "misc.h"



bool SessionParser::Parse(const std::string & path, SessionContainer & container)
{
	return Parse(path.c_str(), container);
}


void SessionParser::SetUsers(Users * pusers)
{
	users = pusers;
}


bool SessionParser::Parse(const char * path, SessionContainer & container)
{
	container.Clear();
	file.open(path, std::ios_base::in | std::ios_base::binary);

	if( !file )
	{
		log << log1 << "SP: cannot open the session file for reading (file: '" << path << "')" << logend;
		return false;
	}

	bool res = Parse(container);

	file.close();

return res;
}



bool SessionParser::Parse(SessionContainer & container)
{
	/*
		file format, each rows:
		session_id(long) user_id(long) remember_me(0|1) time(long) last_time(long)
	*/

	long user_id;

	session.Clear();
	last = file.get();

	log << log2 << "SP: reading sessions from the session file" << logend;

	while( true )
	{
		session.id = ReadLong();

		if( file.eof() )
			break;
		
		user_id             = ReadLong();
		session.remember_me = ReadLong();
		session.time        = ReadLong();
		session.last_time   = ReadLong();

		if( MakeSession(user_id) )
		{
			users->IncrementLoggedUsers();
			container.PushBack(session);
			log << log2 << "SP: read session id: " << session.id << " for user: " << session.puser->name << logend;
		}

		SkipLine();
	}

	log << log2 << "SP: read " << container.Size() << " session(s)" << logend;

return true;
}


bool SessionParser::MakeSession(long user_id)
{
	User * puser = users->GetUser(user_id);

	if( !puser )
	{
		log << log1 << "SP: there is no a user with id: " << user_id << " (skipped)" << logend;
		return false;
	}

	session.puser        = puser;
	session.new_session  = true;
	session.tm_time      = Time(session.time);
	session.tm_last_time = Time(session.last_time);

return true;
}


bool SessionParser::IsWhite(int c)
{
	if( c==' ' || c=='\t' || c==13 )
		return true;

return false;
}

bool SessionParser::IsDigit(int c)
{
	if( c>='0' && c<='9' )
		return true;

return false;
}

void SessionParser::SkipWhite()
{
	while( IsWhite(last) )
		last = file.get();
}


void SessionParser::SkipLine()
{
	while( last != 10 )
		last = file.get();

	last = file.get(); // first character from the new line
}



long SessionParser::ReadLong()
{
long res = 0;
bool is_sign = false;

	SkipWhite();

	if( last == '-' )
	{
		is_sign = true;
		last = file.get();
	}

	while( IsDigit(last) )
	{
		res  = res*10 + (last-'0');
		last = file.get();
	}

	if( is_sign )
		res = -res;

return res;
}

