/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include "system.h"
#include "misc.h"
#include "error.h"
#include "templates/templates.h"
#include "functions/functionbase.h"



void System::SetRequest(Request * prequest)
{
	request = prequest;
}


void System::SetConfig(Config * pconfig)
{
	config = pconfig;
}


void System::SetDb(Db * pdb)
{
	db = pdb;
}


void System::SetSynchro(Synchro * psynchro)
{
	synchro = psynchro;
}




void System::Init()
{
	dirs.SetDb(db);
	dirs.SetRequest(request);
	dirs.SetNotify(&notify);
	dirs.ReadDirs();

	mounts.SetDirs(&dirs);
	mounts.SetDb(db);
	mounts.SetRequest(request);
	mounts.CreateMounts();
	mounts.ReadMounts();

	users.SetRequest(request);
	users.ReadUsers(db);
	users.SetTimeZoneOffset(config->time_zone_offset);
	groups.ReadGroups(db); // !! chwilowe przekazanie argumentu, db bedzie zmienione

	rebus.SetRequest(request);
	rebus.Init();

	notify.SetSynchro(synchro);
	notify.SetRequest(request);
	notify.SetConfig(config);
	notify.SetUsers(&users);
	notify.SetDirs(&dirs);
	notify.Init();

	thumb.SetSynchro(synchro);
	thumb.SetDb(db);
	thumb.SetConfig(config);
	thumb.SetSystem(this);
}


// !! mozna zrobic jakas obsluge kiedy nie mozemy sie redirectnac, np gdy wystapil blad
// !! moze zwracac jakas wartosc?
void System::RedirectTo(const Item & item, const wchar_t * postfix)
{
	request->redirect_to = config->base_url;

	if( item.type == Item::dir )
	{
		// item_id is pointing to a directory
		dirs.MakePath(item.id, request->redirect_to, false);
	}
	else
	{
		// item_id is pointing to a file or a symlink
		if( dirs.MakePath(item.parent_id, request->redirect_to, false) )
			request->redirect_to += item.url;
	}

	if( postfix )
		request->redirect_to += postfix;
}



void System::RedirectTo(long item_id, const wchar_t * postfix)
{
	request->redirect_to = config->base_url;
	Item * pdir = dirs.GetDir(item_id);

	if( pdir )
	{
		// item_id is pointing to a directory
		dirs.MakePath(pdir->id, request->redirect_to, false);
	}
	else
	{
		// item_id is pointing to a file
		DbItemQuery iq;

		iq.SetAllSel(false);
		iq.WhereId(item_id);
		iq.sel_parent_id = true;
		iq.sel_url       = true;

		if( db->GetItem(item_temp, iq) == WINIX_ERR_OK )
		{
			if( dirs.MakePath(item_temp.parent_id, request->redirect_to, false) )
				request->redirect_to += item_temp.url;
		}
		else
		{
			log << log1 << "System: can't redirect: no such item: id: " << item_id << logend;	
		}
	}

	if( postfix )
		request->redirect_to += postfix;
}


void System::RedirectTo(const std::wstring & url)
{
	request->redirect_to = config->base_url;

	if( !url.empty() && url[0] == '/' )
	{
		// absolute path
		request->redirect_to += url;
	}
	else
	{
		// relative path
		if( !request->dir_tab.empty() )
		{
			if( dirs.MakePath(request->dir_tab.back()->id, request->redirect_to, false) )
				request->redirect_to += url;
		}
		else
		{
			request->redirect_to += '/';
			request->redirect_to += url;
		}
	}
}


void System::AddParams(const ParamTab & param_tab, std::wstring & str, bool clear_str)
{
	if( clear_str )
		str.clear();

	for(size_t i=0 ; i<param_tab.size() ; ++i)
	{
		str += '/';
		str += param_tab[i].name;

		if( !param_tab[i].value.empty() )
		{
			str += ':';
			str += param_tab[i].value;
		}
	}
}



void System::RedirectWithFunctionAndParamsTo(const std::wstring & url)
{
	RedirectTo(url);

	if( !request->function )
		return;

	request->redirect_to += '/';
	request->redirect_to += request->function->fun.url;

	AddParams(request->param_tab, request->redirect_to, false);
}


void System::RedirectToLastDir()
{
	if( !request->dir_tab.empty() )
		RedirectTo( *request->dir_tab.back() );
}


void System::RedirectToLastItem()
{
	if( request->last_item )
		RedirectTo( *request->last_item );
}



bool System::CanChangeUser(const Item & item, long new_user_id)
{
	if( !request->session )
		// session must be set
		return false;
	
	if( request->session->puser && request->session->puser->super_user )
		// super user is allowed everything
		return true;
		
	// !! przeciez to prosciej mozna zapisac
	// albo dac od razu return false
	if( item.user_id != new_user_id )
		// only super user can change the owner of an item
		return false;

return true;
}


bool System::CanChangeGroup(const Item & item, long new_group_id)
{
	if( !request->session )
		// session must be set
		return false;

	if( request->session->puser && request->session->puser->super_user )
		// super user is allowed everything
		return true;

	if( item.group_id != new_group_id )
	{
		// user is allowed to change the group only if he is an owner of the item
		// he can change only into a group in which he is a member of, or into a 'no_group'
		
		if( !request->session->puser )
			return false;
			
		if( request->session->puser->id != item.user_id )
			return false;
			
		if( new_group_id == -1 )
			return true;
			
		if( !request->session->puser->IsMemberOf(new_group_id) )
			return false;
			
		// is logged, is the owner of the item, is the member of the new group
	}
	
return true;
}


bool System::CanChangePrivileges(const Item & item, int new_priv)
{
	if( !request->session )
		// session must be set
		return false;

	if( request->session->puser && request->session->puser->super_user )
		// super user is allowed everything
		return true;

	if( item.privileges != new_priv )
	{
		// the owner of an item is allowed to change the privileges		
		
		if( !request->session->puser )
			return false;
			
		if( request->session->puser->id != item.user_id )
			return false;
	}

return true;
}



bool System::HasAccess(const Item & item, int mask)
{
	if( !request->session )
		// session must be set
		return false;

	if( request->session->puser && request->session->puser->super_user )
		// super user is allowed everything
		return true;
	
	if( request->session->puser && request->session->puser->id == item.user_id )
	{
		// the owner
		return ((item.privileges >> 6) & mask) == mask;
	}
	
	if( request->session->puser && request->session->puser->IsMemberOf(item.group_id) )
	{
		// group
		return ((item.privileges >> 3) & mask) == mask;
	}
	
	// others
	
return (item.privileges & mask) == mask;
}


bool System::HasReadAccess(const Item & item)
{
	return HasAccess(item, 4);	
}


bool System::HasWriteAccess(const Item & item)
{
	return HasAccess(item, 2);	
}


bool System::HasReadWriteAccess(const Item & item)
{
	return HasAccess(item, 6);	// r+w
}


bool System::HasReadExecAccess(const Item & item)
{
	if( request->session && request->session->puser && request->session->puser->super_user )
	{
		// there must be at least one 'x' (for the root)
	
		return (item.privileges & 0111) != 0;
	}
	
	return HasAccess(item, 5);	// r+x
}





bool System::HasReadExecAccessToPath(long dir_id)
{
	while( true )
	{
		Item * pdir = dirs.GetDir(dir_id);

		if( !pdir )
			return false;	

		if( !HasReadExecAccess(*pdir) )
			return false;

		dir_id = pdir->parent_id;

		if( dir_id == -1 )
			return true;
	}
}


bool System::HasReadExecAccessToPath(const std::vector<Item*> & dir_tab)
{
	for(size_t i=0 ; i < dir_tab.size() ; ++i)
	{
		if( !HasReadExecAccess(*dir_tab[i]) )
			return false;
	}
	
return true;	
}


bool System::DirsHaveReadExecPerm()
{
	return HasReadExecAccessToPath(request->dir_tab);
}


// if we don't have access we only remove the item from the table
void System::CheckAccessToItems(std::vector<Item> & item_tab)
{
size_t i = 0;

	while( i < item_tab.size() )
	{
		if( !HasReadAccess(item_tab[i]) )
		{
			item_tab.erase(item_tab.begin() + i);
		}
		else
		{
			i += 1;
		}
	}
}



bool System::CanUseHtml(long user_id)
{
	return IsMemberOfGroup(user_id, L"allow_html");
}


bool System::CanUseBBCode(long user_id)
{
	// logged users can use bbcode
	return (user_id != -1);
}


bool System::CanUseRaw(long user_id)
{
	return IsMemberOfGroup(user_id, L"allow_raw");
}



bool System::IsMemberOfGroup(long user_id, const wchar_t * group_name)
{
	User * puser = users.GetUser(user_id);
	
	if( !puser )
		return false;
	
	if( puser->super_user )
		return true; // !! ?? zakladamy ze administrator jest czlonkiem wszystkich grup? dlaczego?
	
	long group = groups.GetGroupId(group_name);
	
	if( group == -1 )
		// there is no such a group
		return false;
	
return puser->IsMemberOf(group);
}





// the path depends on parent_id
bool System::CreateNewFileSimpleFs(Item & item)
{
	bool res = dirs.MakePath(item.parent_id, item.file_path);

	if( res )
	{
		if( !item.file_path.empty() && item.file_path[0] == '/' )
			item.file_path.erase(0, 1);
	}
	else
	{
		log << log1 << "System: CreateNewFileSimpleFs: can't create a path to item.id: " << item.id
		<< ", item.parent_id: " << item.parent_id << logend;
	}

return res;
}




// the path depends on id
bool System::CreateNewFileHashFs(Item & item)
{
wchar_t buffer[50];
wchar_t * hash = buffer;
size_t buffer_len = sizeof(buffer)/sizeof(wchar_t);

	// get 'id' as hexadecimal
	buffer[0] = '0';
	swprintf(buffer+1, buffer_len, L"%lx", (unsigned long)item.id);

	item.file_path.clear();
	
	// make sure that the length is even
	if( (wcslen(hash) & 1) != 0 )
		hash = buffer + 1; // the first character was zero

	for(size_t i=0 ; hash[i] != 0 ; i+=2)
	{
		item.file_path += hash[i];
		item.file_path += hash[i+1];

		if( hash[i+2] != 0 )
			item.file_path += '/';
	}

	// one character more to make sure the path is unique
	// (we can have a directory without the character)
	item.file_path += '_';

return true;
}



// creating item.file_path and item.file_fs
// you should set file_type yourself
// this method uses: item.id, item.url, item.parent_id (for selecting a mountpoint)
bool System::CreateNewFile(Item & item)
{
bool res;

	if( item.type != Item::file )
	{
		log << log1 << "System: CreateNewFile: the item should be a file" << logend;
		return false;
	}

	Mount * pmount = mounts.CalcMount(item.parent_id);

	if( !pmount || pmount->fs != mounts.MountFsHashfs() )
	{
		res = CreateNewFileSimpleFs(item);
		item.file_fs = mounts.MountFsSimplefs();
	}
	else
	{
		res = CreateNewFileHashFs(item);
		item.file_fs = mounts.MountFsHashfs();
	}

	if( res )
		item.file_path += item.url;
	else
		item.file_path.clear();

return res;
}


// making a global file path (in the unix file system)
// you should call CreateNewFile before
bool System::MakeFilePath(const Item & item, std::wstring & path, bool thumb, bool create_dir, int chmod)
{
	path.clear();

	if( config->upload_dir.empty() )
	{
		log << log1 << "System: MakePath: upload_dir is not set in the config file" << logend;
		return false;
	}

	if( item.file_path.empty() || item.file_type == WINIX_ITEM_FILETYPE_NONE )
	{
		log << log1 << "System: MakePath: this item has not a static file" << logend;
		return false;
	}

	path = config->upload_dir;

	if( item.file_fs == mounts.MountFsHashfs() )
		path += L"/hashfs";
	else
		path += L"/simplefs";


	if( thumb )
		path += L"/thumb";
	else
		path += L"/normal";


	if( create_dir && !CreateDirs(path, item.file_path, chmod, true) )
		return false;

	path += '/';
	path += item.file_path;

return true;
}


// item can be a directory, file or a symlink
// if item is a directory then the path will be with a slash at the end
bool System::MakePath(const Item & item, std::wstring & path, bool clear_path)
{
bool res;

	if( clear_path )
		path.clear();

	if( item.type == Item::dir )
	{
		res = dirs.MakePath(item.id, path);
	}
	else
	{
		res = dirs.MakePath(item.parent_id, path);

		if( res )
			path += item.url;
	}

return res;
}




Error System::AddFile(Item & item, int notify_code)
{
	if( item.type != Item::file )
		return WINIX_ERR_FILE_EXPECTED;

	Error status = db->AddItem(item);
	
	if( status == WINIX_ERR_OK )
	{
		log << log2 << "System: added a new file, url: " << item.url << ", id: " << item.id
			<< ", parent_id: " << item.parent_id << logend;

		if( notify_code )
			notify.ItemChanged(notify_code, item);
	}

return status;
}




Error System::EditFile(Item & item, bool with_url, int notify_code)
{
	if( item.type != Item::file )
		return WINIX_ERR_FILE_EXPECTED;

	if( request->session && request->session->puser )
		request->item.modification_user_id = request->session->puser->id;
	else
		request->item.modification_user_id = -1;

	item.SetDateModifyToNow();
	Error status = db->EditItemById(item, with_url);
	
	if( status == WINIX_ERR_OK )
	{
		TemplatesFunctions::pattern_cacher.UpdatePattern(item);

		log << log2 << "System: modified an item" << logend;

		if( notify_code )
			notify.ItemChanged(notify_code, item);
	}


return status;
}


time_t System::LocalTime(time_t gmt_time)
{
int time_offset;

	if( request->session && request->session->puser )
		time_offset = request->session->puser->time_zone_offset;
	else
		time_offset = config->time_zone_offset_guest;

return gmt_time + (time_t)time_offset;
}


tm System::LocalTime(const tm * ptm)
{
time_t t;
tm rtm;

	t = Time(ptm);
	t = LocalTime(t);
	rtm = Time(t);

return rtm;
}


tm System::LocalTime(const tm & ptm)
{
	return LocalTime(&ptm);
}



/*
	return codes:
	ok:
	0 - the link_to is a path to a directory (out_item skipped, out_dir_tab will not be empty)
	1 - the link_to is a path to a file or a symlink (out_item is used, out_dir_tab will not be empty)

	error:
	2 - incorrect link_to
	3 - there is not a root dir
	4 - current_dir_tab was empty

	current_dir_tab can be the same container as out_dir_tab
	link_to can be a relative path (without the first slash)
*/
int System::FollowLink(const std::vector<Item*> & current_dir_tab, const std::wstring & link_to,
					   std::vector<Item*> & out_dir_tab, Item & out_item)
{
	link_to_temp = link_to; // copy the link because it can be from out_item (and will be cleared)
	out_item.Clear();

	int res = dirs.FollowLink(current_dir_tab, link_to_temp, out_dir_tab, name_temp);

	if( res == 1 )
	{
		if( db->GetItem(out_dir_tab.back()->id, name_temp, out_item) == WINIX_ERR_OK )
			return 1;			
		else
			return 2;
	}

return res;
}








bool System::FollowAllLinksDirFound(std::vector<Item*> & out_dir_tab,
									bool follow_dir_default, bool stop_on_link_redirect, bool check_access)
{
	log << log3 << "System: link to a directory: ";
	dirs.LogDir(out_dir_tab);
	log << logend;

	if( check_access && !HasReadExecAccessToPath(out_dir_tab) )
	{
		log << log1 << "System: no access to the directory structure" << logend;
		return false;
	}

	if( !out_dir_tab.back()->link_to.empty() )
	{
		if( follow_dir_default )
		{
			if( !(stop_on_link_redirect && out_dir_tab.back()->link_redirect==1) )
				link_to_temp = out_dir_tab.back()->link_to;
		}
	}

return true;
}



bool System::FollowAllLinksFileOrSymlinkFound(std::vector<Item*> & out_dir_tab, Item & out_item,
											  bool stop_on_link_redirect, bool check_access)
{
	if( out_item.type == Item::symlink )
		log << log3 << "System: link to a symlink: ";
	else
		log << log3 << "System: link to a file: ";

	dirs.LogDir(out_dir_tab);
	log << out_item.url << logend;

	if( check_access && !HasReadExecAccessToPath(out_dir_tab) )
	{
		log << log1 << "System: no access to the directory structure" << logend;
		return false;
	}

	if( out_item.type == Item::symlink )
	{
		if( out_item.link_to.empty() )
		{
			log << log1 << "System: symlink empty" << logend;
			return false;
		}
		else
		{
			if( !check_access || HasReadAccess(out_item) )
			{
				if( !(stop_on_link_redirect && out_item.link_redirect==1) )
					link_to_temp = out_item.link_to;
			}
			else
			{
				log << log1 << "System: no read access to the symlink" << logend;
				return false;
			}
		}
	}

return true;
}






/*
	return codes:
	ok:
	0 - the link_to is a path to a directory (out_item skipped, out_dir_tab will not be empty)
	1 - the link_to is a path to a file      (out_item is used, out_dir_tab will not be empty)
	    (link_to can be a path to a symlink if stop_on_link_redirect is true)

	error:
	2 - incorrect link_to
	3 - there is not a root dir
	4 - current_dir_tab was empty
	5 - limit of symlinks exceeded
	6 - permission denied to a file/symlink or a directory (or a symlink is empty)

	current_dir_tab can be the same container as out_dir_tab
	link_to can be a relative path (without the first slash)

	if follow_dir_default is true then directories with 'default' flags are followed as well

	if stop_on_link_redirect is true then the method stops on a symbolic link (or a directory 
	with 'default' flag set) where the redirection should be done, you should check then whether
	the out_item.back()->link_to is not empty (if the result was 0 ) or
	whether out_item.type is symlink (if the result was 1) 
	to make the redirection
*/
int System::FollowAllLinks(const std::vector<Item*> & current_dir_tab, const std::wstring & link_to,
					       std::vector<Item*> & out_dir_tab, Item & out_item,
						   bool follow_dir_default, bool stop_on_link_redirect, bool check_access)
{
	int res;
	size_t level = 0;
	link_to_temp = link_to;

	if( current_dir_tab.empty() )
		return 4;

	do
	{
		if( ++level > config->symlinks_follow_max )
		{
			log << log1 << "System: the number of maximum symlinks exceeded ("
						<< config->symlinks_follow_max << ")" << logend;

			res = 5;
		}
		else
		{
			res = FollowLink(current_dir_tab, link_to_temp, out_dir_tab, out_item);
			link_to_temp.clear();

			if( res == 0 )
			{
				out_item.Clear();
				res = FollowAllLinksDirFound(out_dir_tab, follow_dir_default, stop_on_link_redirect, check_access) ? 0 : 6;
			}
			else
			if( res == 1 )
			{
				res = FollowAllLinksFileOrSymlinkFound(out_dir_tab, out_item, stop_on_link_redirect, check_access) ? 1 : 6;
			}
			else
			{
				log << log2 << "System: incorrect link: " << link_to << logend;
			}
		}
	}
	while( !link_to_temp.empty() );

return res;
}


// the same as FollowAllLinks but starts from the root directory
// link_to must begin with a slash (or can be empty then the root directory is returned)
int System::FollowAllLinks(const std::wstring & link_to,
						   std::vector<Item*> & out_dir_tab, Item & out_item,
						   bool follow_dir_default, bool stop_on_link_redirect, bool check_access)
{
	if( !link_to.empty() && link_to[0] != '/' )
		return 2;

	if( root_follow_dir_tab.size() != 1 )
		root_follow_dir_tab.resize(1);

	Item * root_dir = dirs.GetRootDir();

	if( !root_dir )
		return 3;

	root_follow_dir_tab[0] = root_dir;

	return FollowAllLinks(root_follow_dir_tab, link_to,
						  out_dir_tab, out_item, follow_dir_default, stop_on_link_redirect, check_access);
}



// the same as FollowAllLinks but operates on request->dir_tab and request->item
// and returns bool
// the method is making a redirection if needed
bool System::FollowAllLinks(const std::wstring & link_to,
							bool follow_dir_default, bool stop_on_link_redirect, bool check_access)
{
	int res = FollowAllLinks(request->dir_tab, link_to, temp_follow_dir_tab, temp_follow_item,
							 follow_dir_default, stop_on_link_redirect, check_access);

	bool ok = (res == 0 || res == 1);

	if( ok )
	{
		request->dir_tab = temp_follow_dir_tab;
		
		if( res == 0 )
		{
			request->is_item = false;
			request->item.Clear();
			request->last_item = request->dir_tab.back();
			
			if( !request->dir_tab.back()->link_to.empty() )
				RedirectTo(request->dir_tab.back()->link_to);

			log << log3 << "System: current directory changed" <<  logend;
		}
		else
		{
			request->is_item   = true;
			request->item      = temp_follow_item;
			request->last_item = &request->item;

			if( request->item.type == Item::symlink )
				RedirectTo(request->item.link_to); // request->item.link_to is not empty

			log << log3 << "System: current directory changed and the new file loaded" <<  logend;
		}

		mounts.CalcCurMount();
	}
	else
	{
		if( res == 5 || res == 6 )
			request->status = WINIX_ERR_PERMISSION_DENIED;
		else
			request->status = WINIX_ERR_NO_ITEM;
	}

return ok;
}



