/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#ifndef headerfilecmslucoresystem
#define headerfilecmslucoresystem

#include <ctime>
#include "dirs.h"
#include "mounts.h"
#include "db/db.h"
#include "request.h"
#include "config.h"
#include "users.h"
#include "groups.h"
#include "rebus.h"
#include "loadavg.h"
#include "synchro.h"
#include "thumb.h"
#include "notify/notify.h"




// file system
class System
{
public:

	// contains current directories tree
	Dirs dirs;

	// mount points
	Mounts mounts;

	// users
	Users users;
	
	// groups
	Groups groups;

	// rebus (captcha)
	Rebus rebus;

	// load averages
	LoadAvg load_avg;

	// notifications (by emails)
	Notify notify;

	// thumbnails (special thread)
	Thumb thumb;

	// the time when the winix starts
	time_t system_start;


	void SetRequest(Request * prequest);
	void SetConfig(Config * pconfig);
	void SetDb(Db * pdb);
	void SetSynchro(Synchro * psynchro);
	void Init();

	void AddParams(const ParamTab & param_tab, std::wstring & str, bool clear_str = true);

	void RedirectTo(const Item & item, const wchar_t * postfix = 0);
	void RedirectTo(long item_id, const wchar_t * postfix = 0);
	void RedirectTo(const std::wstring & url);
	void RedirectWithFunctionAndParamsTo(const std::wstring & url);
	void RedirectToLastDir();
	void RedirectToLastItem(); // redirect to an item if exists or to the last directory

	bool CanChangeUser(const Item & item, long new_user_id);
	bool CanChangeGroup(const Item & item, long new_group_id);
	bool CanChangePrivileges(const Item & item, int new_priv);
	bool HasAccess(const Item & item, int mask);
	bool HasReadAccess(const Item & item);
	bool HasWriteAccess(const Item & item);
	bool HasReadWriteAccess(const Item & item);
	bool HasReadExecAccess(const Item & item);
	bool HasReadExecAccessToPath(long dir_id);
	bool HasReadExecAccessToPath(const std::vector<Item*> & dir_tab);
	bool DirsHaveReadExecPerm();
	void CheckAccessToItems(std::vector<Item> & item_tab);

	bool CanUseHtml(long user_id);
	bool CanUseBBCode(long user_id);
	bool CanUseRaw(long user_id);

	bool IsMemberOfGroup(long user_id, const wchar_t * group_name);

	// creating item.file_path and item.file_fs (the mountpoint where the item is located)
	bool CreateNewFile(Item & item);
	bool MakeFilePath(const Item & item, std::wstring & path, bool thumb = false, bool create_dir = false, int chmod = 0755);

	bool MakePath(const Item & item, std::wstring & path, bool clear_path = true);

	Error AddFile(Item & item, int notify_code = 0);
	Error EditFile(Item & item, bool with_url = true, int notify_code = 0);

	// converting GMT time to local time (different for each user)
	time_t LocalTime(time_t gmt_time);
	tm     LocalTime(const tm * ptm);
	tm     LocalTime(const tm & ptm);

	int FollowLink(const std::vector<Item*> & current_dir_tab, const std::wstring & link_to,
				   std::vector<Item*> & out_dir_tab, Item & out_item);

	int FollowAllLinks(const std::vector<Item*> & current_dir_tab, const std::wstring & link_to,
					   std::vector<Item*> & out_dir_tab, Item & out_item,
					   bool follow_dir_default = false, bool stop_on_link_redirect = false, bool check_access = true);

	// starting from root dir
	int FollowAllLinks(const std::wstring & link_to,
					   std::vector<Item*> & out_dir_tab, Item & out_item,
					   bool follow_dir_default = false, bool stop_on_link_redirect = false, bool check_access = true);

	// using request->dir_tab and request->item
	bool FollowAllLinks(const std::wstring & link_to,
						bool follow_dir_default = false, bool stop_on_link_redirect = false, bool check_access = true);

private:
	Request   * request;
	Config    * config;
	Db        * db;
	Synchro   * synchro;

	Item item_temp;
	std::wstring link_to_temp, name_temp;

	// for FollowAllLinks
	std::vector<Item*> temp_follow_dir_tab;
	std::vector<Item*> root_follow_dir_tab;
	Item temp_follow_item;

	bool CreateNewFileSimpleFs(Item & item);
	bool CreateNewFileHashFs(Item & item);

	bool FollowAllLinksDirFound(std::vector<Item*> & out_dir_tab,
								bool follow_dir_default, bool stop_on_link_redirect, bool check_access);

	bool FollowAllLinksFileOrSymlinkFound(std::vector<Item*> & out_dir_tab, Item & out_item,
										  bool stop_on_link_redirect, bool check_access);
};


#endif

