/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#ifndef headerfile_winix_core_textstream
#define headerfile_winix_core_textstream

#include <string>
#include "misc.h"


/*
	a special class representing a stream buffer
	similar to std::ostringstream

	StringType can be either std::string or std::wstring
	this class doesn't use UTF-8 in any kind
*/
template<class StringType>
class TextStream
{
public:

	typedef typename StringType::value_type CharType;
	typedef typename StringType::value_type char_type;

	void Clear();
	bool Empty() const;
	size_t Size() const;
	void Reserve(size_t len);

	const StringType & Str() const;
	const CharType * CStr() const;

	TextStream & operator<<(const char * str);
	TextStream & operator<<(const std::string * str);
	TextStream & operator<<(const std::string & str);

	TextStream & operator<<(const wchar_t * str);
	TextStream & operator<<(const std::wstring * str);
	TextStream & operator<<(const std::wstring & str);

	TextStream & operator<<(char);
	TextStream & operator<<(wchar_t);
	TextStream & operator<<(int);
	TextStream & operator<<(long);
	TextStream & operator<<(unsigned int);
	TextStream & operator<<(unsigned long);
	TextStream & operator<<(double);
	TextStream & operator<<(const void *);// printing a pointer

	TextStream & Write(const char * buf, size_t len);
	TextStream & Write(const wchar_t * buf, size_t len);
	TextStream & write(const char * buf, size_t len);     // for compatibility with standard library (Ezc uses it)
	TextStream & write(const wchar_t * buf, size_t len);

protected:

	StringType buffer;

};



template<class StringType>
void TextStream<StringType>::Clear()
{
	buffer.clear();
}

template<class StringType>
bool TextStream<StringType>::Empty() const
{
	return buffer.empty();
}

template<class StringType>
size_t TextStream<StringType>::Size() const
{
	return buffer.size();
}

template<class StringType>
void TextStream<StringType>::Reserve(size_t len)
{
	buffer.reserve(len);
}


template<class StringType>
const StringType & TextStream<StringType>::Str() const
{
	return buffer;
}


template<class StringType>
const typename TextStream<StringType>::CharType * TextStream<StringType>::CStr() const
{
	return buffer.c_str();
}



template<class StringType>
TextStream<StringType> & TextStream<StringType>::operator<<(const char * str)
{
	AssignString(str, buffer, false);

return *this;
}


template<class StringType>
TextStream<StringType> & TextStream<StringType>::operator<<(const std::string * str)
{
	AssignString(*str, buffer, false);

return *this;
}


template<class StringType>
TextStream<StringType> & TextStream<StringType>::operator<<(const std::string & str)
{
	AssignString(str, buffer, false);

return *this;
}



template<class StringType>
TextStream<StringType> & TextStream<StringType>::operator<<(const wchar_t * str)
{
	AssignString(str, buffer, false);

return *this;
}


template<class StringType>
TextStream<StringType> & TextStream<StringType>::operator<<(const std::wstring * str)
{
	AssignString(*str, buffer, false);

return *this;
}


template<class StringType>
TextStream<StringType> & TextStream<StringType>::operator<<(const std::wstring & str)
{
	AssignString(str, buffer, false);

return *this;
}



template<class StringType>
TextStream<StringType> & TextStream<StringType>::operator<<(char v)
{
	buffer += v;

return *this;
}


template<class StringType>
TextStream<StringType> & TextStream<StringType>::operator<<(wchar_t v)
{
	buffer += static_cast<CharType>(v);

return *this;
}


template<class StringType>
TextStream<StringType> & TextStream<StringType>::operator<<(int v)
{
wchar_t buf[50];
size_t len = sizeof(buf) / sizeof(wchar_t);

	Toa(v, buf, len);
	AssignString(buf, buffer, false);

return *this;
}


template<class StringType>
TextStream<StringType> & TextStream<StringType>::operator<<(long v)
{
wchar_t buf[50];
size_t len = sizeof(buf) / sizeof(wchar_t);

	Toa(v, buf, len);
	AssignString(buf, buffer, false);

return *this;
}


template<class StringType>
TextStream<StringType> & TextStream<StringType>::operator<<(unsigned int v)
{
wchar_t buf[50];
size_t len = sizeof(buf) / sizeof(wchar_t);

	Toa(v, buf, len);
	AssignString(buf, buffer, false);

return *this;
}


template<class StringType>
TextStream<StringType> & TextStream<StringType>::operator<<(unsigned long v)
{
wchar_t buf[50];
size_t len = sizeof(buf) / sizeof(wchar_t);

	Toa(v, buf, len);
	AssignString(buf, buffer, false);

return *this;
}


template<class StringType>
TextStream<StringType> & TextStream<StringType>::operator<<(double v)
{
char buf[50];

	sprintf(buf, "%f", v);
	AssignString(buf, buffer, false);

return *this;
}


template<class StringType>
TextStream<StringType> & TextStream<StringType>::operator<<(const void * v)
{
wchar_t buf[50];
size_t len = sizeof(buf) / sizeof(wchar_t);

	buf[0] = '0';
	buf[1] = 'x';

	Toa(reinterpret_cast<unsigned long>(v), buf+2, len-2, 16);
	AssignString(buf, buffer, false);

return *this;
}


template<class StringType>
TextStream<StringType> & TextStream<StringType>::Write(const char * buf, size_t len)
{
	AssignString(buf, len, buffer, false);
	
return *this;
}


template<class StringType>
TextStream<StringType> & TextStream<StringType>::write(const char * buf, size_t len)
{
	return Write(buf, len);
}


template<class StringType>
TextStream<StringType> & TextStream<StringType>::Write(const wchar_t * buf, size_t len)
{
	AssignString(buf, len, buffer, false);
	
return *this;
}


template<class StringType>
TextStream<StringType> & TextStream<StringType>::write(const wchar_t * buf, size_t len)
{
	return Write(buf, len);
}

#endif

