/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#ifndef headerfile_winix_db_dbtextstream
#define headerfile_winix_db_dbtextstream

#include <ctime>
#include "core/textstream.h"



/*
	DbTextStream is used as a buffer for creating a database's query
	By default all operators<< escape its string artuments. If you don't want
	to escape an argument you should use a helper function R() (raw argument)
	note: you have to define the function yourself, we do not provide it
	because such a short name would make a mess in namespaces

	sample:
	create a helper function R as follows:

		template<class RawType>
		DbTextStream::RawText<RawType> R(const RawType & par)
		{
			return DbTextStream::RawText<RawType>(par);
		}
	
	now you can use DbTextStream in an easy way:

		DbTextStream query;
		std::string key = "some string";
		query << R("select * from table where key=") << key << R(";");

	in above example only the key is escaped.

	Also with escaping operators<< insert commas between parameters, e.g.:
		
		query << R("insert into table (key1, key2, key3) values (")
			  << key1
			  << key2
			  << key3
			  << R(");");

	between key1 key2 and key3 are commas inserted automatically
*/
class DbTextStream : public TextStream<std::string>
{
public:


	/*
		a helper struct to select a proper operator<<
		(for non-escaping versions of these operators)
	*/
	template<class RawType>
	struct RawText
	{
		const RawType & par;

		RawText(const RawText<RawType> & p) : par(p.par) {}
		RawText(const RawType & p) : par(p) {}
	};


	DbTextStream();

	// extented escaping: adding E character before the first quote e.g. E'string'
	// default: true
	void SetExtented(bool ext);



	/*
		without escaping
	*/

	DbTextStream & PutText(const char *);
	DbTextStream & PutText(const std::string *);
	DbTextStream & PutText(const std::string &);
	DbTextStream & PutText(const wchar_t * str);
	DbTextStream & PutText(const std::wstring * str);
	DbTextStream & PutText(const std::wstring & str);

	/*
		we need this template operator for such calling:
		dbtextstream_object << R("some string");
		"some string" is actually a table (not a pointer)
	*/
	template<size_t str_size>
	DbTextStream & operator<<(const RawText<char [str_size]> & raw)	{ return PutText(raw.par); }

	template<size_t str_size>
	DbTextStream & operator<<(const RawText<wchar_t [str_size]> & raw)	{ return PutText(raw.par); }

	DbTextStream & operator<<(const RawText<const char*> & raw);
	DbTextStream & operator<<(const RawText<const wchar_t*> & raw);
	DbTextStream & operator<<(RawText<const std::string*> raw);
	DbTextStream & operator<<(RawText<const std::wstring*> raw);
	DbTextStream & operator<<(RawText<std::string> raw);
	DbTextStream & operator<<(RawText<std::wstring> raw);
	DbTextStream & operator<<(RawText<char> raw);
	DbTextStream & operator<<(RawText<wchar_t> raw);
	DbTextStream & operator<<(RawText<int> raw);
	DbTextStream & operator<<(RawText<long> raw);
	DbTextStream & operator<<(RawText<unsigned int> raw);
	DbTextStream & operator<<(RawText<unsigned long> raw);
	DbTextStream & operator<<(RawText<double> raw);
	DbTextStream & operator<<(RawText<void*> raw);
	DbTextStream & operator<<(const RawText<tm> & t);


	/*
		with escaping
	*/

	DbTextStream & EBinPutChar(char c);
	DbTextStream & ETextPutChar(char c);
	DbTextStream & ETextPutChar(wchar_t c);

	DbTextStream & EPutText(const char * str);
	DbTextStream & EPutText(const std::string * str);
	DbTextStream & EPutText(const std::string & str);
	DbTextStream & EPutText(const wchar_t * str);
	DbTextStream & EPutText(const std::wstring * str);
	DbTextStream & EPutText(const std::wstring & str);

	DbTextStream & EPutBin(const char * str, size_t len);
	DbTextStream & EPutBin(const std::string * str); 
	DbTextStream & EPutBin(const std::string & str); 

	DbTextStream & operator<<(const char * str);
	DbTextStream & operator<<(const std::string * str);
	DbTextStream & operator<<(const std::string & str);
	DbTextStream & operator<<(const wchar_t * str);
	DbTextStream & operator<<(const std::wstring * str);
	DbTextStream & operator<<(const std::wstring & str);
	DbTextStream & operator<<(char);
	DbTextStream & operator<<(wchar_t);
	DbTextStream & operator<<(int);
	DbTextStream & operator<<(long);
	DbTextStream & operator<<(unsigned int);
	DbTextStream & operator<<(unsigned long);
	DbTextStream & operator<<(double);
	DbTextStream & operator<<(const void *);
	DbTextStream & operator<<(const tm & t);


	static const char * ConvertTime(const tm & t);

private:

	bool was_param;
	bool ext_escape;

};




#endif

