/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2010, Tomasz Sowa
 * All rights reserved.
 * 
 */

#include "tdb.h"
#include "core/log.h"



namespace Ticket
{


bool TDb::IsTicket(long file_id)
{
	PGresult * r = 0;
	bool is = false;
	
	try
	{
		query.Clear();
		query << R("select count(*) from plugins.ticket "
				   "where ticket.file_id=") << file_id << R(";");
		
		r = AssertQuery(query);
		AssertResult(r, PGRES_TUPLES_OK);

		if( Rows(r)==1 && Cols(r)==1 )
			is = AssertValueInt(r, 0, 0) != 0;
	}
	catch(const Error &)
	{
	}
	
	ClearResult(r);

return is;
}


Error TDb::GetTicket(long file_id, Ticket & ticket)
{
	PGresult * r = 0;
	Error status = WINIX_ERR_OK;
	ticket.Clear();
	
	try
	{
		query.Clear();
		query << R("select file_id, param, int_value, str_value from plugins.ticket "
				   "where ticket.file_id=") << file_id << R(";");
		
		r = AssertQuery(query);
		AssertResult(r, PGRES_TUPLES_OK);

		int rows = Rows(r);

		if( rows == 0 )
			throw Error(WINIX_ERR_NO_TICKET);
	
		Ticket::TicketParam par;
		ticket.file_id = file_id;

		int cparam    = AssertColumn(r, "param");
		int cintvalue = AssertColumn(r, "int_value");
		int cstrvalue = AssertColumn(r, "str_value");

		for(int i=0 ; i<rows ; ++i)
		{
			par.param     = AssertValueInt(r, i, cparam);
			par.int_value = AssertValueLong(r, i, cintvalue);
			AssertValueWide(r, i, cstrvalue, par.str_value);

			ticket.par_tab.push_back(par);
		}
	}
	catch(const Error & e)
	{
		status = e;
	}
	
	ClearResult(r);

return status;
}






Error TDb::GetTickets(const std::vector<long> & file_id_tab, std::vector<Ticket> & ticket_tab)
{
	PGresult * r = 0;
	Error status = WINIX_ERR_OK;
	ticket_tab.clear();
	
	if( file_id_tab.empty() )
		return status;

	try
	{
		query.Clear();
		CreateIdList(file_id_tab, file_list);
		query << R("select file_id, param, int_value, str_value from plugins.ticket "
				   "where ticket.file_id in ") << R(file_list) << R(" order by file_id;");

		// !! moze dodac operator <<(std::vector(long)) do query?

		r = AssertQuery(query);
		AssertResult(r, PGRES_TUPLES_OK);

		int rows = Rows(r);
		Ticket::TicketParam par;

		int cfile_id   = AssertColumn(r, "file_id");
		int cparam     = AssertColumn(r, "param");
		int cintvalue  = AssertColumn(r, "int_value");
		int cstrvalue  = AssertColumn(r, "str_value");
		long last_file_id = -1;

		for(int i=0 ; i<rows ; ++i)
		{
			long file_id = AssertValueLong(r, i, cfile_id);

			if( i==0 || last_file_id != file_id )
			{
				ticket_tab.push_back(Ticket());
				ticket_tab.back().file_id = file_id;
				last_file_id = file_id;
			}

			par.param = AssertValueInt(r, i, cparam);
			par.int_value = AssertValueLong(r, i, cintvalue);
			AssertValueWide(r, i, cstrvalue, par.str_value);

			ticket_tab.back().par_tab.push_back(par);
		}
	}
	catch(const Error & e)
	{
		status = e;
	}
	
	ClearResult(r);
	file_list.clear();

return status;
}




Error TDb::AddTicket(const Ticket & ticket)
{
	PGresult * r = 0;
	Error status = WINIX_ERR_OK;
	
	try
	{
		for(size_t i=0 ; i<ticket.par_tab.size() ; ++i)
		{
			query.Clear();
			query << R("insert into plugins.ticket (file_id, param, int_value, str_value) values (")
				  << ticket.file_id
				  << ticket.par_tab[i].param
				  << ticket.par_tab[i].int_value
				  << ticket.par_tab[i].str_value
				  << R(");");
			
			r = AssertQuery(query);
			AssertResult(r, PGRES_COMMAND_OK);
		}
	}
	catch(const Error & e)
	{
		status = e;
	}
	
	ClearResult(r);

return status;
}




Error TDb::RemoveAddTicket(const Ticket & ticket)
{
	Error status = BeginTrans();

	if( status != WINIX_ERR_OK )
		return status;

	status = RemoveTicket(ticket.file_id);

	if( status == WINIX_ERR_OK )
		status = AddTicket(ticket);

	if( status == WINIX_ERR_OK )
		status = CommitTrans();
	else
		RollbackTrans();

return status;
}





Error TDb::RemoveTicket(long file_id)
{
	PGresult * r = 0;
	Error status = WINIX_ERR_OK;
	
	try
	{
		query.Clear();
		query << R("delete from plugins.ticket where file_id=") << file_id << R(";");

		r = AssertQuery(query);
		AssertResult(r, PGRES_COMMAND_OK);

		long rows = AffectedRows(r);

		if( rows > 0 )
			log << log2 << "Db: deleted " << rows << " rows from plugins.ticket" << logend;	
		
	}
	catch(const Error & e)
	{
		status = e;
	}
	
	ClearResult(r);

return status;
}





} // namespace

